"""Common functions for package biobb_analysis.gromacs"""

import re
import shutil
from pathlib import Path, PurePath
from typing import Optional, Union

from biobb_common.command_wrapper import cmd_wrapper
from biobb_common.tools import file_utils as fu


def gmx_check(file_a: str, file_b: str, gmx: str = "gmx") -> bool:
    print("Comparing GROMACS files:")
    print("FILE_A: %s" % str(Path(file_a).resolve()))
    print("FILE_B: %s" % str(Path(file_b).resolve()))
    check_result = "check_result.out"
    cmd = [gmx, "check"]
    if file_a.endswith(".tpr"):
        cmd.append("-s1")
    else:
        cmd.append("-f")
    cmd.append(file_a)
    if file_b.endswith(".tpr"):
        cmd.append("-s2")
    else:
        cmd.append("-f2")
    cmd.append(file_b)
    cmd.append("> check_result.out")
    cmd_wrapper.CmdWrapper(cmd).launch()
    print("Result file: %s" % str(Path(check_result).resolve()))
    with open(check_result) as check_file:
        for line_num, line in enumerate(check_file):
            if not line.rstrip():
                continue
            if line.startswith("Both files read correctly"):
                continue
            if not line.startswith("comparing"):
                print("Discrepance found in line %d: %s" % (line_num, line))
                return False
    return True


def check_energy_path(path, out_log, classname):
    """Checks energy input file"""
    if not Path(path).exists():
        fu.log(classname + ": Unexisting energy input file, exiting", out_log)
        raise SystemExit(classname + ": Unexisting energy input file")
    file_extension = PurePath(path).suffix
    if not is_valid_energy(file_extension[1:]):
        fu.log(
            classname + ": Format %s in energy input file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in energy input file is not compatible" % file_extension[1:]
        )
    # if file input has no path, add cwd because execution is launched on tmp folder
    if PurePath(path).name == path or not PurePath(path).is_absolute():
        path = str(PurePath(Path.cwd()).joinpath(path))
    return path


def check_input_path(path, out_log, classname):
    """Checks input structure file"""
    if not Path(path).exists():
        fu.log(classname + ": Unexisting structure input file, exiting", out_log)
        raise SystemExit(classname + ": Unexisting structure input file")
    file_extension = PurePath(path).suffix
    if not is_valid_structure(file_extension[1:]):
        fu.log(
            classname + ": Format %s in structure input file is not compatible"
            % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in structure input file is not compatible"
            % file_extension[1:]
        )
    # if file input has no path, add cwd because execution is launched on tmp folder
    if PurePath(path).name == path or not PurePath(path).is_absolute():
        path = str(PurePath(Path.cwd()).joinpath(path))
    return path


def check_index_path(path, out_log, classname):
    """Checks index input file"""
    if not path:
        return None
    file_extension = PurePath(path).suffix
    if not is_valid_index(file_extension[1:]):
        fu.log(
            classname + ": Format %s in index input file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in index input file is not compatible" % file_extension[1:]
        )
    # if file input has no path, add cwd because execution is launched on tmp folder
    if PurePath(path).name == path or not PurePath(path).is_absolute():
        path = str(PurePath(Path.cwd()).joinpath(path))
    return path


def check_traj_path(path, out_log, classname):
    """Checks input structure file"""
    if not Path(path).exists():
        fu.log(classname + ": Unexisting trajectory input file, exiting", out_log)
        raise SystemExit(classname + ": Unexisting trajectory input file")
    file_extension = PurePath(path).suffix
    if not is_valid_trajectory(file_extension[1:]):
        fu.log(
            classname + ": Format %s in trajectory input file is not compatible"
            % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in trajectory input file is not compatible"
            % file_extension[1:]
        )
    # if file input has no path, add cwd because execution is launched on tmp folder
    if PurePath(path).name == path or not PurePath(path).is_absolute():
        path = str(PurePath(Path.cwd()).joinpath(path))
    return path


def check_out_xvg_path(path, out_log, classname):
    """Checks if output folder exists and format is xvg"""
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ": Unexisting output folder, exiting", out_log)
        raise SystemExit(classname + ": Unexisting output folder")
    file_extension = PurePath(path).suffix
    if not is_valid_xvg(file_extension[1:]):
        fu.log(
            classname + ": Format %s in output file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in output file is not compatible" % file_extension[1:]
        )
    return path


def check_out_pdb_path(path, out_log, classname):
    """Checks if output folder exists and format is xvg"""
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ": Unexisting output folder, exiting", out_log)
        raise SystemExit(classname + ": Unexisting output folder")
    file_extension = PurePath(path).suffix
    if not is_valid_structure(file_extension[1:]):
        fu.log(
            classname + ": Format %s in output file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in output file is not compatible" % file_extension[1:]
        )
    return path


def check_out_traj_path(path, out_log, classname):
    """Checks if output folder exists and format is correct"""
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ": Unexisting output folder, exiting", out_log)
        raise SystemExit(classname + ": Unexisting output folder")
    file_extension = PurePath(path).suffix
    if not is_valid_trajectory_output(file_extension[1:]):
        fu.log(
            classname + ": Format %s in output file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in output file is not compatible" % file_extension[1:]
        )
    return path


def check_out_str_ens_path(path, out_log, classname):
    """Checks if output folder exists and format is correct"""
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ": Unexisting output folder, exiting", out_log)
        raise SystemExit(classname + ": Unexisting output folder")
    file_extension = PurePath(path).suffix
    if not is_valid_zip(file_extension[1:]):
        fu.log(
            classname + ": Format %s in output file is not compatible" % file_extension[1:],
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in output file is not compatible" % file_extension[1:]
        )
    return path


def get_default_value(key):
    """Gives default values according to the given key"""

    default_values = {
        "instructions_file": "instructions.in",
        "binary_path": "gmx",
        "terms": ["Potential"],
        "selection": "System",
        "xvg": "none",
        "dista": False,
        "method": "linkage",
        "cutoff": 0.1,
        "cluster_selection": "System",
        "fit_selection": "System",
        "center_selection": "System",
        "output_selection": "System",
        "pbc": "mol",
        "center": True,
        "fit": "none",
        "ur": "compact",
        "skip": 1,
        "start": None,
        "end": None,
        "dt": None,
        "ot_str_ens": "pdb",
    }

    return default_values[key]


def get_binary_path(properties, type):
    """Gets binary path"""
    return properties.get(type, get_default_value(type))


def get_terms(properties, out_log, classname):
    """Gets energy terms"""
    terms = properties.get("terms", dict())
    if not terms or not isinstance(terms, list):
        fu.log(classname + ": No terms provided or incorrect format, exiting", out_log)
        raise SystemExit(classname + ": No terms provided or incorrect format")
    if not is_valid_term(terms):
        fu.log(classname + ": Incorrect terms provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect terms provided")
    return properties.get("terms", "")


def get_selection(properties, out_log, classname):
    """Gets selection items"""
    selection = properties.get("selection", get_default_value("selection"))
    if not selection:
        fu.log(
            classname + ": No selection provided or incorrect format, exiting", out_log
        )
        raise SystemExit(classname + ": No selection provided or incorrect format")
    if not is_valid_selection(selection):
        fu.log(classname + ": Incorrect selection provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect selection provided")
    return selection


def get_image_selection(properties, key, out_log, classname):
    """Gets selection items"""
    selection = properties.get(key, get_default_value(key))
    if not selection:
        fu.log(
            classname + ": No selection provided or incorrect format, exiting", out_log
        )
        raise SystemExit(classname + ": No selection provided or incorrect format")
    if not is_valid_selection(selection):
        fu.log(classname + ": Incorrect selection provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect selection provided")
    return selection


def get_selection_index_file(properties, index, key, out_log, classname):
    """Gets selection items from provided index file"""
    pattern = re.compile(r"\[.*\]")
    selection = []
    with open(index, "r") as ndx_file:
        for i, line in enumerate(ndx_file):
            for match in re.finditer(pattern, line):
                selection.append(re.sub(r"[\[\] ]", "", match.group()))
    sel = properties.get(key, get_default_value(key))
    if sel not in selection:
        fu.log(classname + ": Incorrect selection provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect selection provided")
    return sel


def get_pbc(properties, out_log, classname):
    """Gets pbc"""
    pbc = properties.get("pbc", get_default_value("pbc"))
    if not is_valid_pbc(pbc):
        fu.log(classname + ": Incorrect pbc provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect pbc provided")
    return pbc


def get_center(properties, out_log, classname):
    """Gets center"""
    center = properties.get("center", get_default_value("center"))
    if not is_valid_boolean(center):
        fu.log(classname + ": Incorrect center provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect center provided")
    return center


def get_ur(properties, out_log, classname):
    """Gets ur"""
    ur = properties.get("ur", get_default_value("ur"))
    if not is_valid_ur(ur):
        fu.log(classname + ": Incorrect ur provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect ur provided")
    return ur


def get_fit(properties, out_log, classname):
    """Gets fit"""
    fit = properties.get("fit", get_default_value("fit"))
    if not is_valid_fit(fit):
        fu.log(classname + ": Incorrect fit provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect fit provided")
    return fit


def get_skip(properties, out_log, classname):
    """Gets skip"""
    skip = properties.get("skip", get_default_value("skip"))
    if not is_valid_int(skip):
        fu.log(classname + ": Incorrect skip provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect start provided")
    return str(skip)


def get_start(properties, out_log, classname):
    """Gets start"""
    start = properties.get("start", get_default_value("start"))

    if start is None:
        return start
    if not is_valid_int(start):
        fu.log(classname + ": Incorrect start provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect start provided")
    return str(start)


def get_end(properties, out_log, classname):
    """Gets end"""
    end = properties.get("end", get_default_value("end"))
    if end is None:
        return end
    if not is_valid_int(end):
        fu.log(classname + ": Incorrect end provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect end provided")
    return str(end)


def get_dt(properties, out_log, classname):
    """Gets dt"""
    dt = properties.get("dt", get_default_value("dt"))
    if dt is None:
        return dt
    if not is_valid_int(dt):
        fu.log(classname + ": Incorrect dt provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect dt provided")
    return str(dt)


def get_ot_str_ens(properties, out_log, classname):
    """Gets output type"""
    output_type = properties.get("output_type", get_default_value("ot_str_ens"))
    if not is_valid_ot_str_ens(output_type):
        fu.log(classname + ": Incorrect output_type provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect output_type provided")
    return str(output_type)


def get_xvg(properties, out_log, classname):
    """Gets xvg"""
    xvg = properties.get("xvg", get_default_value("xvg"))
    if not is_valid_xvg_param(xvg):
        fu.log(classname + ": Incorrect xvg provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect xvg provided")
    return xvg


def get_dista(properties, out_log, classname):
    """Gets dista"""
    dista = properties.get("dista", get_default_value("dista"))
    if not is_valid_boolean(dista):
        fu.log(classname + ": Incorrect dista provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect dista provided")
    return dista


def get_method(properties, out_log, classname):
    """Gets method"""
    method = properties.get("method", get_default_value("method"))
    if not is_valid_method_param(method):
        fu.log(classname + ": Incorrect method provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect method provided")
    return method


def get_cutoff(properties, out_log, classname):
    """Gets cutoff"""
    cutoff = properties.get("cutoff", get_default_value("cutoff"))
    if not is_valid_float(cutoff):
        fu.log(classname + ": Incorrect cutoff provided, exiting", out_log)
        raise SystemExit(classname + ": Incorrect cutoff provided")
    return str(cutoff)


def is_valid_boolean(val):
    """Checks if given value is boolean"""
    values = [True, False]
    return val in values


def is_valid_float(val):
    """Checks if given value is float"""
    if val and not isinstance(val, float) and not isinstance(val, int):
        return False
    return True


def is_valid_int(val):
    """Checks if given value is int"""
    if val and not isinstance(val, int):
        return False
    return True


def is_valid_method_param(met):
    """Checks if method is compatible with GROMACS"""
    methods = ["linkage", "jarvis-patrick", "monte-carlo", "diagonalization", "gromos"]
    return met in methods


def is_valid_structure(ext):
    """Checks if structure format is compatible with GROMACS"""
    formats = ["tpr", "gro", "g96", "pdb", "brk", "ent"]
    return ext in formats


def is_valid_index(ext):
    """Checks if structure format is compatible with GROMACS"""
    formats = ["ndx"]
    return ext in formats


def is_valid_trajectory(ext):
    """Checks if trajectory format is compatible with GROMACS"""
    formats = ["xtc", "trr", "cpt", "gro", "g96", "pdb", "tng"]
    return ext in formats


def is_valid_trajectory_output(ext):
    """Checks if trajectory format is compatible with GROMACS"""
    formats = ["xtc", "trr", "gro", "g96", "pdb", "tng"]
    return ext in formats


def is_valid_energy(ext):
    """Checks if energy format is compatible with GROMACS"""
    formats = ["edr"]
    return ext in formats


def is_valid_xvg(ext):
    """Checks if file is XVG"""
    formats = ["xvg"]
    return ext in formats


def is_valid_zip(ext):
    """Checks if file is ZIP"""
    formats = ["zip"]
    return ext in formats


def is_valid_xvg_param(ext):
    """Checks xvg parameter"""
    formats = ["xmgrace", "xmgr", "none"]
    return ext in formats


def is_valid_ot_str_ens(ext):
    """Checks if output type for structure ensemble is correct"""
    formats = ["gro", "g96", "pdb"]
    return ext in formats


def is_valid_pbc(pbc):
    """Checks pbc parameter"""
    values = ["none", "mol", "res", "atom", "nojump", "cluster", "whole"]
    return pbc in values


def is_valid_ur(ur):
    """Checks ur parameter"""
    values = ["rect", "tric", "compact"]
    return ur in values


def is_valid_fit(fit):
    """Checks fit parameter"""
    values = [
        "none",
        "rot+trans",
        "rotxy+transxy",
        "translation",
        "transxy",
        "progressive",
    ]
    return fit in values


def is_valid_term(iterms):
    """Checks if term is correct"""
    cterms = [
        "Angle",
        "Proper-Dih.",
        "Improper-Dih.",
        "LJ-14",
        "Coulomb-14",
        "LJ-(SR)",
        "Coulomb-(SR)",
        "Coul.-recip.",
        "Position-Rest.",
        "Potential",
        "Kinetic-En.",
        "Total-Energy",
        "Temperature",
        "Pressure",
        " Constr.-rmsd",
        "Box-X",
        "Box-Y",
        " Box-Z",
        "Volume",
        "Density",
        "pV",
        "Enthalpy",
        "Vir-XX",
        "Vir-XY",
        "Vir-XZ",
        "Vir-YX",
        "Vir-YY",
        "Vir-YZ",
        "Vir-ZX",
        "Vir-ZY",
        "Vir-ZZ",
        "Pres-XX",
        "Pres-XY",
        "Pres-XZ",
        "Pres-YX",
        "Pres-YY",
        "Pres-YZ",
        "Pres-ZX",
        "Pres-ZY",
        "Pres-ZZ",
        "#Surf*SurfTen",
        "Box-Vel-XX",
        "Box-Vel-YY",
        "Box-Vel-ZZ",
        "Mu-X",
        "Mu-Y",
        "Mu-Z",
        "T-Protein",
        "T-non-Protein",
        "Lamb-Protein",
        "Lamb-non-Protein",
    ]
    return all(elem in cterms for elem in iterms)


def is_valid_selection(ext):
    """Checks if selection is correct"""
    formats = [
        "System",
        "Protein",
        "Protein-H",
        "C-alpha",
        "Backbone",
        "MainChain",
        "MainChain+Cb",
        "MainChain+H",
        "SideChain",
        "SideChain-H",
        "Prot-Masses",
        "non-Protein",
        "Water",
        "SOL",
        "non-Water",
        "Ion",
        "NA",
        "CL",
        "Water_and_ions",
        "DNA",
        "RNA",
        "Protein_DNA",
        "Protein_RNA",
        "Protein_DNA_RNA",
        "DNA_RNA",
        "DPPC",
        "DMPC",
        "POPG",
        "POPA",
        "POPC",
        "POPE",
        "DMTAP",
        "POPS"
    ]
    return ext in formats


def copy_instructions_file_to_container(instructions_file, unique_dir):
    shutil.copy2(instructions_file, unique_dir)


def remove_tmp_files(list, remove_tmp, out_log):
    """Removes temporal files generated by the wrapper"""
    if remove_tmp:
        tmp_files = list
        removed_files = [f for f in tmp_files if fu.rm(f)]
        fu.log("Removed: %s" % str(removed_files), out_log)


def process_output_trjconv_str_ens(
    tmp_folder, output_file, output_dir, glob_pattern, out_log
):
    tmp_fl = list(Path(tmp_folder).glob(glob_pattern))
    if not tmp_fl:
        tmp_fl = list(Path(tmp_folder).glob("frame*.pdb"))

    files_list = []
    for file_name in tmp_fl:
        files_list.append(file_name)

    # adding files from temporary folder to zip
    fu.zip_list(output_file, files_list, out_log)

    shutil.copy2(output_file, output_dir)


def _from_string_to_list(input_data: Optional[Union[str, list[str]]]) -> list[str]:
    """
    Converts a string to a list, splitting by commas or spaces. If the input is already a list, returns it as is.
    Returns an empty list if input_data is None.

    Parameters:
        input_data (str, list, or None): The string, list, or None value to convert.

    Returns:
        list: A list of string elements or an empty list if input_data is None.
    """
    if input_data is None:
        return []

    if isinstance(input_data, list):
        # If input is already a list, return it
        return input_data

    # If input is a string, determine the delimiter based on presence of commas
    delimiter = "," if "," in input_data else " "
    items = input_data.split(delimiter)

    # Remove whitespace from each item and ignore empty strings
    processed_items = [item.strip() for item in items if item.strip()]

    return processed_items



from . import gmx_cluster
from . import gmx_energy
from . import gmx_image
from . import gmx_rgyr
from . import gmx_rms
from . import gmx_trjconv_str
from . import gmx_trjconv_str_ens
from . import gmx_trjconv_trj

name = "gromacs"
__all__ = ["gmx_cluster", "gmx_energy", "gmx_image", "gmx_rgyr", "gmx_rms", "gmx_trjconv_str", "gmx_trjconv_str_ens", "gmx_trjconv_trj"]



#!/usr/bin/env python3

"""Module containing the GMX Energy class and the command line interface."""

import argparse
from pathlib import PurePath
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger

from biobb_analysis.gromacs.common import (
    _from_string_to_list,
    check_energy_path,
    check_out_xvg_path,
    copy_instructions_file_to_container,
    get_binary_path,
    get_default_value,
    get_terms,
    get_xvg,
)


class GMXEnergy(BiobbObject):
    """
    | biobb_analysis GMXEnergy
    | Wrapper of the GROMACS energy module for extracting energy components from a given GROMACS energy file.
    | `GROMACS energy <http://manual.gromacs.org/current/onlinehelp/gmx-energy.html>`_ extracts energy components from an energy file. The user is prompted to interactively select the desired energy terms.

    Args:
        input_energy_path (str): Path to the input EDR file. File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/gromacs/energy.edr>`_. Accepted formats: edr (edam:format_2330).
        output_xvg_path (str): Path to the XVG output file. File type: output. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/reference/gromacs/ref_energy.xvg>`_. Accepted formats: xvg (edam:format_2030).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **xvg** (*str*) - ("none") XVG plot formatting. Values: xmgrace, xmgr, none.
            * **terms** (*list*) - (["Potential"]) Energy terms. Values: Angle, Proper-Dih., Improper-Dih., LJ-14, Coulomb-14, LJ-\(SR\), Coulomb-\(SR\), Coul.-recip., Position-Rest., Potential, Kinetic-En., Total-Energy, Temperature, Pressure,  Constr.-rmsd, Box-X, Box-Y,  Box-Z, Volume, Density, pV, Enthalpy, Vir-XX, Vir-XY, Vir-XZ, Vir-YX, Vir-YY, Vir-YZ, Vir-ZX, Vir-ZY, Vir-ZZ, Pres-XX, Pres-XY, Pres-XZ, Pres-YX, Pres-YY,  Pres-YZ, Pres-ZX, Pres-ZY, Pres-ZZ, #Surf*SurfTen, Box-Vel-XX, Box-Vel-YY, Box-Vel-ZZ, Mu-X, Mu-Y, Mu-Z, T-Protein, T-non-Protein, Lamb-Protein, Lamb-non-Protein.
            * **binary_path** (*str*) - ("gmx") Path to the GROMACS executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None) Container path definition.
            * **container_image** (*str*) - ('gromacs/gromacs:2022.2') Container image definition.
            * **container_volume_path** (*str*) - ('/tmp') Container volume path definition.
            * **container_working_dir** (*str*) - (None) Container working directory definition.
            * **container_user_id** (*str*) - (None) Container user_id definition.
            * **container_shell_path** (*str*) - ('/bin/bash') Path to default shell inside the container.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_analysis.gromacs.gmx_energy import gmx_energy
            prop = {
                'xvg': 'xmgr',
                'terms': ['Potential', 'Pressure']
            }
            gmx_energy(input_energy_path='/path/to/myEnergyFile.edr',
                        output_xvg_path='/path/to/newXVG.xvg',
                        properties=prop)

    Info:
        * wrapped_software:
            * name: GROMACS energy
            * version: >=2019.1
            * license: LGPL 2.1
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(
        self, input_energy_path, output_xvg_path, properties=None, **kwargs
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {
            "in": {"input_energy_path": input_energy_path},
            "out": {"output_xvg_path": output_xvg_path},
        }

        # Properties specific for BB
        self.xvg = properties.get("xvg", "none")
        self.terms = _from_string_to_list(properties.get("terms", ["Potential"]))
        self.instructions_file = get_default_value("instructions_file")
        self.properties = properties

        # Properties common in all GROMACS BB
        self.binary_path = get_binary_path(properties, "binary_path")

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.io_dict["in"]["input_energy_path"] = check_energy_path(
            self.io_dict["in"]["input_energy_path"], out_log, self.__class__.__name__
        )
        self.io_dict["out"]["output_xvg_path"] = check_out_xvg_path(
            self.io_dict["out"]["output_xvg_path"], out_log, self.__class__.__name__
        )
        self.xvg = get_xvg(self.properties, out_log, self.__class__.__name__)
        self.terms = get_terms(self.properties, out_log, self.__class__.__name__)

    def create_instructions_file(self):
        """Creates an input file using the properties file settings"""
        instructions_list = []
        # different path if container execution or not
        if self.container_path:
            self.instructions_file = str(
                PurePath(self.container_volume_path).joinpath(self.instructions_file)
            )
        else:
            self.instructions_file = str(
                PurePath(fu.create_unique_dir()).joinpath(self.instructions_file)
            )
        # self.instructions_file = str(PurePath(fu.create_unique_dir()).joinpath(self.instructions_file))
        fu.create_name(prefix=self.prefix, step=self.step, name=self.instructions_file)

        for t in self.terms:
            instructions_list.append(t)

        # create instructions file
        with open(self.instructions_file, "w") as mdp:
            for line in instructions_list:
                mdp.write(line.strip() + "\n")

        return self.instructions_file

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`GMXEnergy <gromacs.gmx_energy.GMXEnergy>` gromacs.gmx_energy.GMXEnergy object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # create instructions file
        self.create_instructions_file()

        # if container execution, copy intructions file to container
        if self.container_path:
            copy_instructions_file_to_container(
                self.instructions_file, self.stage_io_dict.get("unique_dir")
            )

        self.cmd = [
            self.binary_path,
            "energy",
            "-f",
            self.stage_io_dict["in"]["input_energy_path"],
            "-o",
            self.stage_io_dict["out"]["output_xvg_path"],
            "-xvg",
            self.xvg,
            "<",
            self.instructions_file,
        ]

        # Run Biobb block
        self.run_biobb()

        # Copy files to host
        self.copy_to_host()

        self.tmp_files.extend(
            [
                # self.stage_io_dict.get("unique_dir", ""),
                str(str(PurePath(self.instructions_file).parent)),
            ]
        )
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return self.return_code


def gmx_energy(
    input_energy_path: str,
    output_xvg_path: str,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Execute the :class:`GMXEnergy <gromacs.gmx_energy.GMXEnergy>` class and
    execute the :meth:`launch() <gromacs.gmx_energy.GMXEnergy.launch>` method."""

    return GMXEnergy(
        input_energy_path=input_energy_path,
        output_xvg_path=output_xvg_path,
        properties=properties,
        **kwargs,
    ).launch()

    gmx_energy.__doc__ = GMXEnergy.__doc__


def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(
        description="Extracts energy components from a given GROMACS energy file.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument("--config", required=False, help="Configuration file")

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument(
        "--input_energy_path",
        required=True,
        help="Path to the input EDR file. Accepted formats: edr.",
    )
    required_args.add_argument(
        "--output_xvg_path",
        required=True,
        help="Path to the XVG output file. Accepted formats: xvg.",
    )

    args = parser.parse_args()
    args.config = args.config or "{}"
    properties = settings.ConfReader(config=args.config).get_prop_dic()

    # Specific call of each building block
    gmx_energy(
        input_energy_path=args.input_energy_path,
        output_xvg_path=args.output_xvg_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Module containing the Cpptraj Input class and the command line interface."""
import argparse
from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_analysis.ambertools.common import get_default_value, check_in_path


class CpptrajInput(BiobbObject):
    """
    | biobb_analysis CpptrajInput
    | Wrapper of the Ambertools Cpptraj module for performing multiple analysis and trajectory operations of a given trajectory.
    | Cpptraj (the successor to ptraj) is the main program in Ambertools for processing coordinate trajectories and data files. The parameter names and defaults are the same as the ones in the official `Cpptraj manual <https://amber-md.github.io/cpptraj/CPPTRAJ.xhtml>`_.

    Args:
        input_instructions_path (str): Path of the instructions file. File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/ambertools/cpptraj.in>`_. Accepted formats: in (edam:format_2033).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **binary_path** (*str*) - ("cpptraj") Path to the cpptraj executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_analysis.ambertools.cpptraj_input import cpptraj_input
            prop = { }
            cpptraj_input(input_instructions_path='/path/to/myInstructions.in',
                            properties=prop)

    Info:
        * wrapped_software:
            * name: Ambertools Cpptraj
            * version: >=20.0
            * license: GNU
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, input_instructions_path=None, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Properties specific for BB
        self.input_instructions_path = input_instructions_path
        self.input_top_path = kwargs.get('input_top_path')
        self.input_traj_path = kwargs.get('input_traj_path')
        self.output_cpptraj_path = kwargs.get('output_cpptraj_path')
        self.properties = properties
        self.binary_path = properties.get('binary_path', 'cpptraj')

        # Check the properties
        self.check_properties(properties)

    def create_instrucions_file(self):
        """ Creates an input file using paths provideed in the configuration file (only used for test purposes) """
        instructions_list = []
        output_instructions_path = fu.create_name(prefix=self.prefix, step=self.step, name=get_default_value("instructions_file"))

        instructions_list.append('parm ' + str(self.input_top_path))
        instructions_list.append('trajin ' + str(self.input_traj_path))
        instructions_list.append('trajout ' + str(self.output_cpptraj_path) + ' ' + get_default_value("format"))

        with open(output_instructions_path, 'w') as mdp:
            for line in instructions_list:
                mdp.write(line.strip() + '\n')

        return output_instructions_path

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`CpptrajInput <ambertools.cpptraj_input.CpptrajInput>` ambertools.cpptraj_input.CpptrajInput object."""

        # Get local loggers from launchlogger decorator

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        output_instructions_path = self.create_instrucions_file() if not self.input_instructions_path else self.input_instructions_path
        check_in_path(output_instructions_path, self.out_log, self.__class__.__name__)

        # create cmd and launch execution
        self.cmd = [self.binary_path, '-i', output_instructions_path]

        # Run Biobb block
        self.run_biobb()

        # Copy files to host
        self.copy_to_host()

        return self.return_code


def cpptraj_input(input_instructions_path: str, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`CpptrajInput <ambertools.cpptraj_input.CpptrajInput>` class and
    execute the :meth:`launch() <ambertools.cpptraj_input.CpptrajInput.launch>` method."""

    return CpptrajInput(input_instructions_path=input_instructions_path,
                        properties=properties, **kwargs).launch()

    cpptraj_input.__doc__ = CpptrajInput.__doc__


def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Performs multiple analysis and trajectory operations of a given trajectory.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('--config', required=False, help='Configuration file')

    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('--input_instructions_path', required=True, help='Path of the instructions file.')

    args = parser.parse_args()
    args.config = args.config or "{}"
    properties = settings.ConfReader(config=args.config).get_prop_dic()

    # Specific call of each building block
    cpptraj_input(input_instructions_path=args.input_instructions_path,
                  properties=properties)


if __name__ == '__main__':
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="biobb_analysis",
    version="5.1.0",
    author="Biobb developers",
    author_email="genis.bayarri@irbbarcelona.org",
    description="Biobb_analysis is the Biobb module collection to perform analysis of molecular dynamics simulations.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="Bioinformatics Workflows BioExcel Compatibility",
    url="https://github.com/bioexcel/biobb_analysis",
    project_urls={
        "Documentation": "http://biobb-analysis.readthedocs.io/en/latest/",
        "Bioexcel": "https://bioexcel.eu/",
    },
    packages=setuptools.find_packages(exclude=["docs", "test"]),
    package_data={"biobb_analysis": ["py.typed"]},
    install_requires=["biobb_common==5.1.0"],
    python_requires=">=3.9",
    entry_points={
        "console_scripts": [
            "cpptraj_average = biobb_analysis.ambertools.cpptraj_average:main",
            "cpptraj_bfactor = biobb_analysis.ambertools.cpptraj_bfactor:main",
            "cpptraj_convert = biobb_analysis.ambertools.cpptraj_convert:main",
            "cpptraj_dry = biobb_analysis.ambertools.cpptraj_dry:main",
            "cpptraj_image = biobb_analysis.ambertools.cpptraj_image:main",
            "cpptraj_mask = biobb_analysis.ambertools.cpptraj_mask:main",
            "cpptraj_rgyr = biobb_analysis.ambertools.cpptraj_rgyr:main",
            "cpptraj_rms = biobb_analysis.ambertools.cpptraj_rms:main",
            "cpptraj_rmsf = biobb_analysis.ambertools.cpptraj_rmsf:main",
            "cpptraj_slice = biobb_analysis.ambertools.cpptraj_slice:main",
            "cpptraj_snapshot = biobb_analysis.ambertools.cpptraj_snapshot:main",
            "cpptraj_strip = biobb_analysis.ambertools.cpptraj_strip:main",
            "gmx_cluster = biobb_analysis.gromacs.gmx_cluster:main",
            "gmx_energy = biobb_analysis.gromacs.gmx_energy:main",
            "gmx_image = biobb_analysis.gromacs.gmx_image:main",
            "gmx_rgyr = biobb_analysis.gromacs.gmx_rgyr:main",
            "gmx_rms = biobb_analysis.gromacs.gmx_rms:main",
            "gmx_trjconv_str_ens = biobb_analysis.gromacs.gmx_trjconv_str_ens:main",
            "gmx_trjconv_str = biobb_analysis.gromacs.gmx_trjconv_str:main",
            "gmx_trjconv_trj = biobb_analysis.gromacs.gmx_trjconv_trj:main",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
    ],
)



#!/usr/bin/env python3

"""Module containing the GMX TrjConvStr class and the command line interface."""
import argparse
from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_analysis.gromacs.common import get_binary_path, check_input_path, check_traj_path, check_index_path, get_selection_index_file, get_selection, check_out_str_ens_path, get_skip, get_start, get_end, get_dt, get_ot_str_ens, process_output_trjconv_str_ens


class GMXTrjConvStrEns(BiobbObject):
    """
    | biobb_analysis GMXTrjConvStrEns
    | Wrapper of the GROMACS trjconv module for extracting an ensemble of frames containing a selection of atoms from GROMACS compatible trajectory files.
    | GROMACS trjconv module can convert trajectory files in many ways. See the `GROMACS trjconv <http://manual.gromacs.org/documentation/2018/onlinehelp/gmx-trjconv.html>`_ official documentation for further information.

    Args:
        input_traj_path (str): Path to the GROMACS trajectory file. File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/gromacs/trajectory.trr>`_. Accepted formats: xtc (edam:format_3875), trr (edam:format_3910), cpt (edam:format_2333), gro (edam:format_2033), g96 (edam:format_2033), pdb (edam:format_1476), tng (edam:format_3876).
        input_top_path (str): Path to the GROMACS input topology file. File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/gromacs/topology.tpr>`_. Accepted formats: tpr (edam:format_2333), gro (edam:format_2033), g96 (edam:format_2033), pdb (edam:format_1476), brk (edam:format_2033), ent (edam:format_1476).
        input_index_path (str) (Optional): Path to the GROMACS index file. File type: input. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/data/gromacs/index.ndx>`_. Accepted formats: ndx (edam:format_2033).
        output_str_ens_path (str): Path to the output file. File type: output. `Sample file <https://github.com/bioexcel/biobb_analysis/raw/master/biobb_analysis/test/reference/gromacs/ref_trjconv.str.ens.zip>`_. Accepted formats: zip (edam:format_3987).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **selection** (*str*) - ("System") Group where the trjconv will be performed. If **input_index_path** provided, check the file for the accepted values. Values: System (all atoms in the system), Protein (all protein atoms), Protein-H (protein atoms excluding hydrogens), C-alpha (C-alpha atoms), Backbone (protein backbone atoms: N; C-alpha and C), MainChain (protein main chain atoms: N; C-alpha; C and O; including oxygens in C-terminus), MainChain+Cb (protein main chain atoms including C-beta), MainChain+H (protein main chain atoms including backbone amide hydrogens and hydrogens on the N-terminus), SideChain (protein side chain atoms: that is all atoms except N; C-alpha; C; O; backbone amide hydrogens and oxygens in C-terminus and hydrogens on the N-terminus), SideChain-H (protein side chain atoms excluding all hydrogens), Prot-Masses (protein atoms excluding dummy masses), non-Protein (all non-protein atoms), Water (water molecules), SOL (water molecules), non-Water (anything not covered by the Water group), Ion (any name matching an Ion entry in residuetypes.dat), NA (all NA atoms), CL (all CL atoms), Water_and_ions (combination of the Water and Ions groups), DNA (all DNA atoms), RNA (all RNA atoms), Protein_DNA (all Protein-DNA complex atoms), Protein_RNA (all Protein-RNA complex atoms), Protein_DNA_RNA (all Protein-DNA-RNA complex atoms), DNA_RNA (all DNA-RNA complex atoms).
            * **skip** (*int*) - (1) [0~10000|1] Only write every nr-th frame.
            * **start** (*int*) - (0) [0~10000|1] Time of first frame to read from trajectory (default unit ps).
            * **end** (*int*) - (0) [0~10000|1] Time of last frame to read from trajectory (default unit ps).
            * **dt** (*int*) - (0) [0~10000|1] Only write frame when t MOD dt = first time (ps).
            * **output_name** (*str*) - ("output") File name for ensemble of output files.
            * **output_type** (*str*) - ("pdb") File type for ensemble of output files. Values: gro (Contains a molecular structure in Gromos87 format), g96 (Can be a GROMOS-96 initial/final configuration file or a coordinate trajectory file or a combination of both), pdb (Molecular structure files in the protein databank file format).
            * **binary_path** (*str*) - ("gmx") Path to the GROMACS executable binary.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None) Container path definition.
            * **container_image** (*str*) - ('gromacs/gromacs:2022.2') Container image definition.
            * **container_volume_path** (*str*) - ('/tmp') Container volume path definition.
            * **container_working_dir** (*str*) - (None) Container working directory definition.
            * **container_user_id** (*str*) - (None) Container user_id definition.
            * **container_shell_path** (*str*) - ('/bin/bash') Path to default shell inside the container.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_analysis.gromacs.gmx_trjconv_str_ens import gmx_trjconv_str_ens
            prop = {
                'selection': 'System',
                'start': 0,
                'end': 10,
                'dt': 1
            }
            gmx_trjconv_str_ens(input_traj_path='/path/to/myStructure.trr',
                                input_top_path='/path/to/myTopology.tpr',
                                output_str_ens_path='/path/to/newStructureEnsemble.zip',
                                input_index_path='/path/to/myIndex.ndx',
                                properties=prop)

    Info:
        * wrapped_software:
            * name: GROMACS trjconv
            * version: >=2019.1
            * license: LGPL 2.1
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, input_traj_path, input_top_path, output_str_ens_path,
                 input_index_path=None, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {
            "in": {"input_traj_path": input_traj_path, "input_top_path": input_top_path, "input_index_path": input_index_path},
            "out": {"output_str_ens_path": output_str_ens_path}
        }

        # Properties specific for BB
        self.selection = properties.get('selection', "System")
        self.skip = properties.get('skip', 1)
        self.start = properties.get('start', 0)
        self.end = properties.get('end', 0)
        self.dt = properties.get('dt', 0)
        self.output_name = properties.get('output_name', "output")
        self.output_type = properties.get('output_type', "pdb")
        self.properties = properties

        # Properties common in all GROMACS BB
        self.binary_path = get_binary_path(properties, 'binary_path')

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.io_dict["in"]["input_traj_path"] = check_traj_path(self.io_dict["in"]["input_traj_path"], out_log, self.__class__.__name__)
        self.io_dict["in"]["input_top_path"] = check_input_path(self.io_dict["in"]["input_top_path"], out_log, self.__class__.__name__)
        self.io_dict["in"]["input_index_path"] = check_index_path(self.io_dict["in"]["input_index_path"], out_log, self.__class__.__name__)
        self.io_dict["out"]["output_str_ens_path"] = check_out_str_ens_path(self.io_dict["out"]["output_str_ens_path"], out_log, self.__class__.__name__)
        if not self.io_dict["in"]["input_index_path"]:
            self.selection = get_selection(self.properties, out_log, self.__class__.__name__)
        else:
            self.selection = get_selection_index_file(self.properties, self.io_dict["in"]["input_index_path"], 'selection', out_log, self.__class__.__name__)
        self.skip = get_skip(self.properties, out_log, self.__class__.__name__)
        self.start = get_start(self.properties, out_log, self.__class__.__name__)
        self.end = get_end(self.properties, out_log, self.__class__.__name__)
        self.dt = get_dt(self.properties, out_log, self.__class__.__name__)
        self.output_name = self.properties.get('output_name', 'output')
        self.output_type = get_ot_str_ens(self.properties, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`GMXTrjConvStrEns <gromacs.gmx_trjconv_str_ens.GMXTrjConvStrEns>` gromacs.gmx_trjconv_str_ens.GMXTrjConvStrEns object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        # standard input
        self.io_dict['in']['stdin_file_path'] = fu.create_stdin_file(f'{self.selection}')
        self.stage_files()

        # if container execution, output to container_volume_path, else to unique_dir
        if self.container_path:
            output = self.container_volume_path + '/' + self.output_name + '.' + self.output_type
        else:
            output = self.stage_io_dict.get("unique_dir", "") + '/' + self.output_name + '.' + self.output_type

        self.cmd = [self.binary_path, 'trjconv',
                    '-f', self.stage_io_dict["in"]["input_traj_path"],
                    '-s', self.stage_io_dict["in"]["input_top_path"],
                    '-skip', self.skip,
                    '-b', self.start,
                    '-dt', self.dt,
                    '-sep',
                    '-o', output]

        # checking 'end' gromacs 'bug'
        if not str(self.end) == "0":
            self.cmd.append('-e')
            self.cmd.append(self.end)

        if self.stage_io_dict["in"].get("input_index_path"):
            self.cmd.extend(['-n', self.stage_io_dict["in"]["input_index_path"]])

        # Add stdin input file
        self.cmd.append('<')
        self.cmd.append(self.stage_io_dict["in"]["stdin_file_path"])

        # Run Biobb block
        self.run_biobb()

        # Copy files to host
        self.copy_to_host()

        if self.container_path:
            process_output_trjconv_str_ens(self.stage_io_dict['unique_dir'],
                                           self.io_dict["out"]["output_str_ens_path"],
                                           self.stage_io_dict.get("unique_dir"),
                                           self.output_name + '*', self.out_log)
        else:
            process_output_trjconv_str_ens(self.stage_io_dict.get("unique_dir"),
                                           self.stage_io_dict["out"]["output_str_ens_path"],
                                           self.io_dict["out"]["output_str_ens_path"],
                                           self.output_name + '*.pdb', self.out_log)

        self.tmp_files.extend([
            # self.stage_io_dict.get("unique_dir", ""),
            self.io_dict['in'].get("stdin_file_path", "")
        ])
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return self.return_code


def gmx_trjconv_str_ens(input_traj_path: str, input_top_path: str, output_str_ens_path: str, input_index_path: Optional[str] = None, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`GMXTrjConvStrEns <gromacs.gmx_trjconv_str_ens.GMXTrjConvStrEns>` class and
    execute the :meth:`launch() <gromacs.gmx_trjconv_str_ens.GMXTrjConvStrEns.launch>` method."""

    return GMXTrjConvStrEns(input_traj_path=input_traj_path,
                            input_top_path=input_top_path,
                            output_str_ens_path=output_str_ens_path,
                            input_index_path=input_index_path,
                            properties=properties).launch()

    gmx_trjconv_str_ens.__doc__ = GMXTrjConvStrEns.__doc__


def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Extracts an ensemble of frames containing a selection of atoms from GROMACS compatible trajectory files.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('--config', required=False, help='Configuration file')

    # Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('--input_traj_path', required=True, help='Path to the GROMACS trajectory file. Accepted formats: xtc, trr, cpt, gro, g96, pdb, tng.')
    required_args.add_argument('--input_top_path', required=True, help='Path to the GROMACS input topology file. Accepted formats: tpr, gro, g96, pdb, brk, ent.')
    parser.add_argument('--input_index_path', required=False, help="Path to the GROMACS index file. Accepted formats: ndx.")
    required_args.add_argument('--output_str_ens_path', required=True, help='Path to the output file. Accepted formats: zip.')

    args = parser.parse_args()
    args.config = args.config or "{}"
    properties = settings.ConfReader(config=args.config).get_prop_dic()

    # Specific call of each building block
    gmx_trjconv_str_ens(input_traj_path=args.input_traj_path,
                        input_top_path=args.input_top_path,
                        output_str_ens_path=args.output_str_ens_path,
                        input_index_path=args.input_index_path,
                        properties=properties)


if __name__ == '__main__':
    main()
